package de.unikoblenz.fgbks.base.utils.boundary.impl;

import static de.unikoblenz.fgbks.base.utils.boundary.BoundType.EXCLUSIVE;
import static de.unikoblenz.fgbks.base.utils.boundary.BoundType.INCLUSIVE;

import de.unikoblenz.fgbks.base.utils.boundary.AbstractGrowingBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.BoundaryParseException;
import java.time.LocalDateTime;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.apache.commons.lang3.Validate;

public class DateBoundary extends AbstractGrowingBoundary<LocalDateTime> {

  public static final String dateGroupName = "datetime";
  private static final String datePatternString =
      "^date and time\\(\"(?<"
          + dateGroupName
          + ">(?:19|20)\\d{2}-(?:0[1-9]|1[0-2])-(?:0[1-9]|[12]\\d|3[01])T(?:[01]\\d|2[0-3]):[0-5]\\d:[0-5]\\d)\"\\)$";
  public static final Pattern datePattern = Pattern.compile(datePatternString);

  public DateBoundary(String text) {
    super(text, LocalDateTime.class);
  }

  @Override
  protected LocalDateTime getMinValue() {
    return LocalDateTime.MAX;
  }

  @Override
  protected LocalDateTime getMaxValue() {
    return LocalDateTime.MIN;
  }

  @Override
  protected void parse(String text) {
    try {
      text = Validate.notNull(text).trim();
      if (text.isEmpty()) {
        upperBound = getMaxValue();
        lowerBound = getMinValue();
        lowerBoundType = INCLUSIVE;
        upperBoundType = INCLUSIVE;
      } else if (text.startsWith("date")) {
        // Exactly
        Matcher matcher = datePattern.matcher(text);
        if (!matcher.find()) {
          throw new IllegalArgumentException();
        } else {
          String dateTime = matcher.group(dateGroupName);
          lowerBoundType = INCLUSIVE;
          upperBoundType = INCLUSIVE;
          lowerBound = upperBound = LocalDateTime.parse(dateTime);
        }
      } else if (text.startsWith("<")) {
        // Before
        Matcher matcher = datePattern.matcher(text.replace('<', '\0').trim());
        if (!matcher.find()) {
          throw new IllegalArgumentException();
        } else {
          String dateTime = matcher.group(dateGroupName);
          lowerBoundType = INCLUSIVE;
          upperBoundType = EXCLUSIVE;
          lowerBound = getMinValue();
          upperBound = LocalDateTime.parse(dateTime);
        }
      } else if (text.startsWith(">")) {
        // After
        Matcher matcher = datePattern.matcher(text.replace('>', '\0').trim());
        if (!matcher.find()) {
          throw new IllegalArgumentException();
        } else {
          String dateTime = matcher.group(dateGroupName);
          lowerBoundType = EXCLUSIVE;
          upperBoundType = INCLUSIVE;
          lowerBound = LocalDateTime.parse(dateTime);
          upperBound = getMaxValue();
        }
      } else if (text.startsWith("[")) {
        // Between
        String[] textDVals = text.replace('[', '\0').replace(']', '\0').trim().split("\\.\\.");
        Matcher matcher1 = datePattern.matcher(textDVals[0]);
        Matcher matcher2 = datePattern.matcher(textDVals[1]);
        if (!matcher1.find() & !matcher2.find()) {
          throw new IllegalArgumentException();
        } else {
          String dateTime1 = matcher1.group(dateGroupName);
          String dateTime2 = matcher2.group(dateGroupName);
          lowerBoundType = INCLUSIVE;
          upperBoundType = INCLUSIVE;
          lowerBound = LocalDateTime.parse(dateTime1);
          upperBound = LocalDateTime.parse(dateTime2);
        }
      } else {
        throw new IllegalArgumentException();
      }
    } catch (Exception e) {
      throw new BoundaryParseException("Date value " + text + " couldn't be parsed.");
    }
  }
}
